-- fourbit_adder
-- This entity adds two 4-bit numbers together.
LIBRARY IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all; -- needed for the unsigned type;


-- DECLARATION of design entity.
entity fourbit_adder is
  generic (
    WIDTH : positive := 4);
  port( 
    a, b : in STD_LOGIC_VECTOR(WIDTH-1 downto 0);
    s    : out STD_LOGIC_VECTOR(WIDTH-1 downto 0);
    cout : out STD_LOGIC
  );
end fourbit_adder;


-- DESCRIPTION of architecture.

-- Architecture #1: Structural
architecture MY_STRUCTURAL of fourbit_adder is
-- will use four 1-bit full adders, chained together.
component full_adder
  port(
    x, y, cin: in std_logic;
    sum, cout: out std_logic
  );
end component;
signal c0, c1, c2, c3 : STD_LOGIC;
-- NOTE: this structural description does not make use of the generic WIDTH yet;
begin
  c0 <= '0';
  FA0: full_adder port map (x=>a(0), y=>b(0), cin=>c0, sum=>s(0), cout=>c1);
  FA1: full_adder port map (x=>a(1), y=>b(1), cin=>c1, sum=>s(1), cout=>c2);
  FA2: full_adder port map (x=>a(2), y=>b(2), cin=>c2, sum=>s(2), cout=>c3);
  FA3: full_adder port map (x=>a(3), y=>b(3), cin=>c3, sum=>s(3), cout=>cout);
end MY_STRUCTURAL;


-- Architecture #2: Behavioral
architecture MY_BEHAVIORAL1 of fourbit_adder is
begin
  process(a, b)
    variable temp : unsigned(WIDTH downto 0);
  begin
    temp := resize(unsigned(a), WIDTH+1)+resize(unsigned(b), WIDTH+1);
    s    <= std_logic_vector(temp(WIDTH-1 downto 0));    
    cout <= temp(WIDTH);
  end process;
end MY_BEHAVIORAL1;


-- Architecture #3: Behavioral
architecture MY_BEHAVIORAL2 of fourbit_adder is
  signal temp : unsigned(WIDTH downto 0);
begin 
  temp <= resize(unsigned(a), WIDTH+1)+resize(unsigned(b), WIDTH+1);
  s    <= std_logic_vector(temp(WIDTH-1 downto 0));    
  cout <= temp(WIDTH);
end MY_BEHAVIORAL2;
